;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Loading this file installs CG/JS.
;; Please see readme.txt for more information
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(in-package :user)

(eval-when (compile eval load)
  (require :shell)
  (use-package :excl.shell))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; validate files

(let ((platform-file
       (directory
	(merge-pathnames
	 "PLATFORM-*"
	 (path-pathname *load-pathname*)))))
  (or platform-file
      (error "Could not find the PLATFORM file."))
  
  (when (> (length platform-file) 1)
    (error "There is more than one PLATFORM file: ~a." platform-file))
  (setq platform-file (car platform-file))
  
  (or (=~ "^PLATFORM-(.*)" (file-namestring platform-file))
      (error "Cannot parse the PLATFORM file."))
  
  (let ((expected-platform $1)
	(current-platform
	 (format nil "~a~a" (symbol-name sys::.os.) #+smp "smp" #-smp "")))
    (or (string-equal expected-platform current-platform)
	(error
	 "The current platform (~a) does not match the downloaded version (~a)."
	 current-platform expected-platform)))
  ;; We are on the correct platform, fall through...
  )

;; HTML files needed for the operation of CG/JS
;; (this variable's value is generated by build_cgjs.sh)
(defvar *html-files* '("cgjs.html" "ssl-not-found.html" "browser-not-supported.html" ))

(defvar *extra-files*)
(setq *extra-files* nil)

(dolist (file *html-files*)
  (setq file (merge-pathnames file *load-pathname*))
  (or (probe-file file)
      (error "~a does not exist." file))
  (push file *extra-files*))
;; All *html-files* exist...

;; These two are directories but the pathname is specified as
;; a file, so merging works below.
(defvar *cg2js-directory* (merge-pathnames "code.cg2js" *load-pathname*))
(defvar *idejs-directory* (merge-pathnames "code.idejs" *load-pathname*))

(dolist (d (list *cg2js-directory* *idejs-directory*))
  (or (probe-file d)
      (error "~a does not exist." d)))

;; MD5 checksums of the incoming files
;; (this variable's value is generated by build_cgjs.sh)
(defvar *checksums* 
'(
(#xf295fcff25a43d36666112a79d923096 . "./code.cg2js/cg-bitmap-pane-clipboard.fasl")
(#xd22aff6e52c0ffe6298a48e8fb9e8a16 . "./code.cg2js/cg-drag-and-drop.fasl")
(#xb6ba85b0c70574e689f8ba6f7bde76aa . "./code.cg2js/cg-toolbar.fasl")
(#x26fdcdbc53449a7eb47e6142f17bd379 . "./code.cg2js/cg-chart-or-plot.fasl")
(#x545af0d3a4fcf6ca995f0393bd408c05 . "./code.cg2js/cg-rich-edit-pane-printing.fasl")
(#x1358f7e02ec01978d02da76d17882ab4 . "./code.cg2js/cg-class-grid.fasl")
(#xe38551d2396c5224f301709fdc6dcf19 . "./code.cg2js/cg-tray.fasl")
(#xe3a2c2ddb1e9b45f954b0183880dc5d0 . "./code.cg2js/cg-gesture-emulation.fasl")
(#x27a269f96799cc9b794f479474dbd001 . "./code.cg2js/cg-class-support.fasl")
(#x139e7c268cf6aeb28222fb724beca2c2 . "./code.cg2js/cg-multi-line-lisp-text.fasl")
(#x6a6048134580c96ed84586f41fe74e9f . "./code.cg2js/cg-dropping-outline.fasl")
(#x37213ebeda107a459e65b270523cc198 . "./code.cg2js/cg-rich-edit-pane.fasl")
(#x30e9916408eeb7611ac415dc9fc8c6d8 . "./code.cg2js/cg-choice-list.fasl")
(#x7346c78c00fcd747c003769e8a763267 . "./code.cg2js/cg-grid-widget-drag-and-drop.fasl")
(#xb9d801a2cd33404d601a06e62f514c36 . "./code.cg2js/cg-picture-widget-palette.fasl")
(#x384e13d10664fe45e22dfb3aeb8cd69f . "./code.cg2js/cg-timer.fasl")
(#x55569a7439bc9ca587604ebc25430c94 . "./code.cg2js/cg-directory-dialog.fasl")
(#x5fcb7a43ad2ee6f32fbe20d6a9e76190 . "./code.cg2js/cg-multi-picture-button-tooltip.fasl")
(#x4d845b0a3312634838266176b165fff2 . "./code.cg2js/cg-outline-edit-in-place.fasl")
(#x57739943dd436c317892a1e99b4c0af9 . "./code.cg2js/cg-pixmap-rotate.fasl")
(#x79aa6e0446593e26eeee5993229ba8a7 . "./code.cg2js/cg-html-widget.fasl")
(#x92e9d70489dff21779c06640bbedca90 . "./code.cg2js/cg-selected-object.fasl")
(#x58f971e57f2a9f4b43493962197c45ae . "./code.cg2js/cg-lamp.fasl")
(#x0c4d6c8622c351ab27a018b33b1f9576 . "./code.cg2js/cg-scroll-bar.fasl")
(#x4802033be3f31c9f662d8d9c05050cb8 . "./code.cg2js/cg-cursor-pixmap.fasl")
(#x7942f645820a664f675b13af70bdb05e . "./code.cg2js/cg-message-dialog.fasl")
(#x6414163d1c44fa3fae3e316497a179b8 . "./code.cg2js/cg-dde-utils.fasl")
(#x2ca2ee0594e3ed9c0fe43e00097776ef . "./code.cg2js/cg-group-box.fasl")
(#x0d7e4f1333a3141662560f8fd4b38772 . "./code.cg2js/cg-find-string-dialog.fasl")
(#x24eb9d103647105bab2b0ff6908bd738 . "./code.cg2js/cg-split-bar.fasl")
(#xefbca847fae1973b7e9fe4f1244cd12c . "./code.cg2js/cg-radio-button.fasl")
(#x152533e3f3fbe0397e655a7990e665b8 . "./code.cg2js/cg-base.fasl")
(#x1e358dcb60d6a09901f0593fd67e60d2 . "./code.cg2js/cg-paren-matching.fasl")
(#x8b764c00ebe05bb4c6f4f027f024e7da . "./code.cg2js/cg-graphics-context.fasl")
(#x13d220814b17fd3cc7450f746f763bc3 . "./code.cg2js/cg-keyboard-shortcuts.fasl")
(#x9cac74d0ee875048561bfeaacbab6e09 . "./code.cg2js/cg-scroll-bar-mixin.fasl")
(#xbf2510e4fa3f6fbb874e4f32c6abf2de . "./code.cg2js/cg-plot-widget.fasl")
(#xa3d3d8ddaa38912690c78ab99361dd4e . "./code.cg2js/cg-dev.fasl")
(#x1d9bdcf84f9f69bb8fe10f9327bbb361 . "./code.cg2js/cg-text-edit-pane.fasl")
(#x27fa11a5c169f650217fd2f31d034eb2 . "./code.cg2js/cg-tab-control.fasl")
(#x4c7a0bbb881a296804f2d4775e3619e6 . "./code.cg2js/cg-fill-texture.fasl")
(#xdf2e6e9eca939a62eb9bc27b43b67949 . "./code.cg2js/cg-menu-tooltip.fasl")
(#x3b9e6a06c9875a254191226e5262e7e6 . "./code.cg2js/cg-trackbar.fasl")
(#x930f925136e4e57e7f875f39fd0f5be3 . "./code.cg2js/cg-nodes.fasl")
(#xe9ddd725dce3926a30b496b797043941 . "./code.cg2js/cg-common-control.fasl")
(#x0382deba6c3b9d899d6e777695d82bae . "./code.cg2js/cg-list-view.fasl")
(#xf6b7d06b344270e13b203826fa7c2665 . "./code.cg2js/cg-rich-edit.fasl")
(#x5efa6bfae568781457332c31d773f186 . "./code.cg2js/cg-project-window.fasl")
(#xf7d7ece257b8f4475fea529ef1b0b8e7 . "./code.cg2js/cg-item-list.fasl")
(#xce928112bee32a27968fa01efa35d9f5 . "./code.cg2js/cg-multi-picture-button-drag-and-drop.fasl")
(#x7c94e490398bd2cfd0319159d8a5497e . "./code.cg2js/cg-get-position.fasl")
(#xf2511081de812ce71ae3061f419f8a2b . "./code.cg2js/cg-bitmap-pane.fasl")
(#x3d6ca2785ca80cef78beffc832eac695 . "./code.cg2js/cg-drawable.fasl")
(#x14ac5e1b70f52f7652db740ea150720b . "./code.cg2js/cg-lettered-menu.fasl")
(#xb074dfdce9863228535bd92fa41192ca . "./code.cg2js/cg-multi-line-editable-text.fasl")
(#x591496342f7def6ac03ffeb1ecc75d12 . "./code.cg2js/cg-html-dialog.fasl")
(#xd33b84b94dc7ba2b2d8099594715143c . "./code.cg2js/cg-object-editor-layout.fasl")
(#x628bedefa6b1860baaf333728a331914 . "./code.cg2js/cg-utility-dialog.fasl")
(#xf2b0acf9aaacd826832ca734368c458b . "./code.cg2js/cg-pixmap-widget.fasl")
(#x1bca36aafc0a21c8b0e7049d3362285b . "./code.cg2js/cg-property.fasl")
(#x94a47e582509922d8588e069611ab64b . "./code.cg2js/cg-menu.fasl")
(#x0600f33ee8ff80f6bbd6f5218948e1fd . "./code.cg2js/cg-clipboard.fasl")
(#x23d685fa7af6d41acd8f9e24cfe739ef . "./code.cg2js/cg-toggling-widget.fasl")
(#x78ce616aede2acfc5bc7aba8dc2403da . "./code.cg2js/cg-rich-edit-pane-clipboard.fasl")
(#x2559b5d18c8eb5fa947f33d02a3ce2af . "./code.cg2js/cg-template-string.fasl")
(#xed5c56bf580e9bb84a232b1f2873f01f . "./code.cg2js/cg-drag-and-drop-image.fasl")
(#xadf2a2187a12c1879f7c47405533ee0d . "./code.cg2js/cg-os-window.fasl")
(#x126ef42c4cdc8ecee5407a63878fe8fc . "./code.cg2js/cg-grid-widget.fasl")
(#x121f1127add582745b0246accd4ad7b8 . "./code.cg2js/cg-picture-widget.fasl")
(#xc073755b61844f38be5e9f08da0ca79a . "./code.cg2js/cg-text-edit-pane-mark.fasl")
(#x79dc7e3f818edd9c81675e56a88e33cb . "./code.cg2js/cg-static-text.fasl")
(#xbaf93bdf6897bb31b67dada961a82de5 . "./code.cg2js/cg-clipboard-stack.fasl")
(#x6351213a90d5d06c6d4df6cf4224cc47 . "./code.cg2js/cg-text-widget.fasl")
(#x0ddfdf2b8b6cd80e00569b53e7a4b241 . "./code.cg2js/cg-calendar.fasl")
(#x467a53857e5ac20418a9cd9abf0bc809 . "./code.cg2js/cg-check-box.fasl")
(#x03f3eefb22673e89ba8b0b634f37a938 . "./code.cg2js/cg-acache.fasl")
(#xe525686262f6c472bc1e48d062068231 . "./code.cg2js/cg-wrap-string.fasl")
(#x73ea1e6c5df1d368917585d6496807d0 . "./code.cg2js/cg-bitmap-stream.fasl")
(#xe5fa5b6110bd4c7d78e296a726d3effe . "./code.cg2js/cg-font-dialog.fasl")
(#xac3dd2a360476cd3584c40e981dc6141 . "./code.cg2js/cg-outline.fasl")
(#xe73216906c152ce1ca7148ba387d3c40 . "./code.cg2js/cg-shortcut-menu.fasl")
(#x612d7c17e6ac6e27c5e9a891a4b22d0d . "./code.cg2js/cg-status-bar.fasl")
(#x751bfdba8c5ae2f9bbe294f049f1692e . "./code.cg2js/cg-multi-picture-button.fasl")
(#x52394fcc38c7e8ad3dc0a5bb272668ae . "./code.cg2js/cg-hotspot.fasl")
(#xa02d306cbf073372c43336b69b520715 . "./code.cg2js/cg-scaling-stream.fasl")
(#x1e77c786d40bf41bde03d21d09058b05 . "./code.cg2js/cg-icon-pixmap.fasl")
(#xf54834673af546a704c553eadcc913f8 . "./code.cg2js/cg-file-dialog.fasl")
(#xf804aaefba208eb52b7a3b0ca34c564c . "./code.cg2js/cg-color-dialog.fasl")
(#x284fd6f11a6d7cdade5347d63210c1b1 . "./code.cg2js/cg-get-pixmap.fasl")
(#xb062555c50854ee99722bdbb6a65011a . "./code.cg2js/cg-object-editor.fasl")
(#x56d5b30a91affc0aadb6b85761491be7 . "./code.cg2js/cg-outline-drag-and-drop.fasl")
(#x05f0cac486cb79846e8912d83589e2f0 . "./code.cg2js/cg-pixmap-file-io.fasl")
(#xbe89a177116764c66a978f7a6fc88068 . "./code.cg2js/cg-pixmap.fasl")
(#x7f7502355b248f82df1a6ef123481c3c . "./code.cg2js/cg-intersect.fasl")
(#xdbe67a74f46693133aa62e45447beb87 . "./code.cg2js/cg-scrolling-static-text.fasl")
(#x84b4a4133a646df38ea34135200ea7d6 . "./code.cg2js/cg-icon.fasl")
(#x50858b9fa5ced12d54f66e000345d7b0 . "./code.cg2js/cg-yes-no-string.fasl")
(#x1bf8ff80caf1fa6892714b758ed9d353 . "./code.cg2js/cg-text-or-combo.fasl")
(#x7d28949fc4a3d18037bce02210520e35 . "./code.cg2js/cg.fasl")
(#x260e81edfe4eea50fc81912ff061e232 . "./code.cg2js/cg-dialog-item.fasl")
(#xcabd9f14f600166670e3f22149d5a16b . "./code.cg2js/cg-drawable-clipboard.fasl")
(#x3386d3921d1ff68b3ceb9a099bd062c1 . "./code.cg2js/cg-caret.fasl")
(#xa6b6311899779074d7d832e756bad93c . "./code.cg2js/cg-editable-text.fasl")
(#xa15a13e4b0a2bf3d3902c8ae44818547 . "./code.cg2js/cg-choose-printer.fasl")
(#x334a61fb935efa726de215b95b4ccf50 . "./code.cg2js/cg-yes-no-list.fasl")
(#xdbd5d4e6ede73b6dde0fa4c4d0e36bad . "./code.cg2js/cg-intersect-posbox.fasl")
(#x017e3de504fd0ec964b5e0cdea160131 . "./code.cg2js/cg-tooltip.fasl")
(#x1011ccd11fd4bfa557e37b2cee022d0f . "./code.cg2js/cg-class-slot-grid.fasl")
(#x88d17f414672aca29f3027a7bf8b3a18 . "./code.cg2js/cg-combo-box.fasl")
(#x5d3bbd270d5fd77c4afafa18577112dc . "./code.cg2js/cg-curve.fasl")
(#x84a330a313448b00f507e9b20bd7086c . "./code.cg2js/cg-web-browser.fasl")
(#x1ee261325197f62c1f19d1cfc084bdd4 . "./code.cg2js/cg-printing.fasl")
(#x63f79db46fe0b25d639b65e9ab439f0c . "./code.cg2js/cg-lisp-widget.fasl")
(#x3606219c58b2ce5ccc8a89e7521b8139 . "./code.cg2js/cg-sample-file-menu.fasl")
(#x9aee1b12b666066a6f4b5996a2c2d276 . "./code.cg2js/cg-palette.fasl")
(#xf5d025485c940d4822dc6dd8070b8b55 . "./code.cg2js/cg-os-widget.fasl")
(#x9008ebe1e2c9e481d894f81683e40ccd . "./code.cg2js/cg-directory-dialog-os.fasl")
(#xe6dc32337f334946dc04eab2905824ef . "./code.cg2js/cg-pixmap-printing.fasl")
(#xf58530c67564e3548800e4d8f7973bd0 . "./code.cg2js/cg-button.fasl")
(#x736dcefee27485b043a7c1c6eaf77ed0 . "./code.cg2js/cg-edit-in-place.fasl")
(#x92be29d6385d0946b0bad8c7ca4a2b94 . "./code.cg2js/cg-text-edit-pane-file-io.fasl")
(#x4caa9fae9decac10a6b8e42cc2000f4e . "./code.cg2js/cg-comtab.fasl")
(#x52e772edd62ece624c8cd2d25948f59a . "./code.cg2js/cg-string-dialog.fasl")
(#x7c928391ab90068a0914a80858a6f866 . "./code.cg2js/cg-header-control.fasl")
(#xd0b9a4bdecb012709602cca7acc50207 . "./code.cg2js/cg-clipboard-pixmap.fasl")
(#xcf51b259a572d7ac007fb4d212036783 . "./code.cg2js/cg-up-down-control.fasl")
(#x970a9ff71a6b620a58e964db38e34481 . "./code.cg2js/cg-layout.fasl")
(#xc23f8ae553ed477d5a183eccfd6b89fa . "./code.cg2js/cg-chart-widget.fasl")
(#x6493b51bbd58f08a15b4e8281d5a4eb1 . "./code.cg2js/cg-lisp-text.fasl")
(#x720b879f1e9c6eca7b90751c15be515f . "./code.cg2js/cg-lisp-edit-pane.fasl")
(#x9b1f13284224fc900217f9fbbf3e9448 . "./code.cg2js/cg-progress-indicator.fasl")
(#x31da46fd9d1720dd16c266c585af866d . "./code.idejs/ide-inspector.fasl")
(#x60581fa3d143ea72d50ad29f8baa93eb . "./code.idejs/ide-grapher.fasl")
(#x65e1d7a9ae0557b6e01d3d3f99caded8 . "./code.idejs/ide-base.fasl")
(#xee55bf58475c2bab685521441549dcac . "./code.idejs/ide-keyshort.fasl")
(#x538cb7e5bd99f348b095ee1ec87a3e87 . "./code.idejs/ide-dev.fasl")
(#x5c640399c1ca7d1d64ab3c84fae13c32 . "./code.idejs/ide-class-browser.fasl")
(#xdd5eed2cee9a7fef456aa72badc86168 . "./code.idejs/ide-package-list.fasl")
(#x243706e186ee3f955caad4471a7bd00a . "./code.idejs/ide-intro.fasl")
(#x2ea5ff8eca8faa4011d1665bb0bddd68 . "./code.idejs/ide-class-grapher.fasl")
(#x606852bc8b6f4aeb401d2947da0350d0 . "./code.idejs/ide-window-list.fasl")
(#xd9ff25e407c2a0d66eb6f6b81263c426 . "./code.idejs/ide-find-definitions.fasl")
(#xf1bb87fa4ac9dc58deff76b05ca391b4 . "./code.idejs/ide-trace.fasl")
(#x58118c5fed4ed696313c4ede9a4b3ca0 . "./code.idejs/ide-editor.fasl")
(#x076051417a0bbf0fcd3de6904cafbdff . "./code.idejs/ide-patch-dialog.fasl")
(#x85b0a706a3e3e6560e4cd75c1a80f154 . "./code.idejs/ide-project.fasl")
(#x0c2a146de9b5aab0130fc31c52d6df4f . "./code.idejs/ide.fasl")
(#xdb88c007a28e8ea1ac9265ad1b517368 . "./code.idejs/ide-apropos.fasl")
(#x1fe3e7e6777022280c46d5148a2b385a . "./code.idejs/ide-acache.fasl")
(#x713f5d2619f8cd49ab5b1f1561feb6cc . "./code.idejs/ide-clipboard.fasl")
(#x0c15c151d6a6dce91b17741e5b13ec11 . "./code.idejs/ide-process-browser.fasl")
(#x938abeb2b65511d73205415a2be0b25d . "./code.idejs/ide-find-in-files.fasl")
(#x0866afcfd742ba991a7e4820c92a6270 . "./code.idejs/ide-form.fasl")
(#x63def8f83b3e479298a7cba7cbea2c99 . "./code.idejs/ide-navigator.fasl")
(#xe396744af4d34e05e734175c54f8cafc . "./cgjs.html")
(#x34779e5d4bbb71d45bf63f88fac54100 . "./ssl-not-found.html")
(#x90c6d2a65504d45266ad9fa3a49be2f9 . "./browser-not-supported.html"))
 )

(let ((files-ok
       (dolist (checksum *checksums* t)
	 (when (/= (car checksum)
		   (md5-file
		    (merge-pathnames (cdr checksum) *load-pathname*)))
	   (return nil)))))
  (cond
   ((not files-ok)
    (error "The extracted files failed the checksum test.  ~
Please download and/or extract again."))
   (t
    (format t "Checksums on files are OK...~%"))))

;; We have validated all the files are there...

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; check for needed patches

(eval-when (compile eval load) (require :build))

(defvar *required-patches* '(#+macosx ("build" . 3)))

(dolist (required-patch *required-patches*)
  (let ((existing-patch (assoc (car required-patch)
			    (cdr (assoc :system sys::*patches*))
			    :test #'string=)))
    (when (null existing-patch)
      (error "Could not find patch info for the ~s module."
	     (car required-patch)))
    (when (not (>= (second existing-patch) (cdr required-patch)))
      ;; The build patch number is NOT >= to the one we require
      (error "~
Expected version ~d of ~s but have version ~d.  Please update ~
your lisp with update.~a and then do the CG/JS installation again."
	     (cdr required-patch) (car required-patch)
	     (second existing-patch)
	     #+mswindows "exe"
	     #-mswindows "sh"))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; copy files

(defun copy (source dest)
  (cond
   ((file-directory-p source)
    (format t "Copy directory ~a~%" (enough-namestring source))
    (copy-directory source dest :quiet t))
   (t
    (format t "Copy file ~a~%" (enough-namestring source))
    (sys:copy-file source dest :verbose nil))))

(defvar *sys* (translate-logical-pathname (pathname "sys:")))

(cond
 ((excl::pathname-equalp (truename *sys*)
			 (path-pathname *load-truename*))
  (format t "Files are already in place, no copying requiring.~%"))
 (t ;; copy files, replacing them with new ones if they already exist
  (let (dest)
    (dolist (from *extra-files*)
      (setq dest (merge-pathnames (file-namestring from) *sys*))
      (when (probe-file dest) (delete-file dest))
      (copy from dest))
    
    (dolist (from (list *cg2js-directory* *idejs-directory*))
      (setq dest (merge-pathnames (file-namestring from) *sys*))
      ;; now make it a directory pathname
      (setq dest (pathname-as-directory dest))

      (when (probe-file dest) (delete-directory-and-files dest))
      (copy (pathname-as-directory from) dest)))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; update develenv.cl to load CG/JS before the original version of CG

(defvar *setup-cgjs-loading* "sys:setup-cgjs-loading.cl")

;; Because users might use an ANSI lisp to install, have to make sure
;; the resulting file can be loaded into an ANSI and Modern lisp.
(with-open-file (s *setup-cgjs-loading* :direction :output
		 :if-exists :supersede)
  (format s "~
;; This file was created by install-cgjs.cl.
;; Please do not edit it.

")
  (dolist (dir (list "code.cg2js/" "code.idejs/"))
    (format s "
(let ((path (make-pathname
	     :type \"fasl\"
	     :directory (pathname-directory
			 (merge-pathnames
			  ~s
			  (translate-logical-pathname \"sys:\"))))))
  (format t \";; *require-search-list*: adding: ~~s~%\" path)
  (push path sys:*require-search-list*))

"
	    dir)))

(defvar *develenv.cl* (pathname "sys:develenv.cl"))

(when (not (probe-file *develenv.cl*))
  (setf (file-contents *develenv.cl*)
    ";; This file is loaded before images are rebuilt with update.sh/update.exe

"))

(let ((contents (file-contents *develenv.cl*)))
  (cond
   ((match-re
     (format nil "^#\\|\\|# \\(load \"sys:~a"
	     (file-namestring *setup-cgjs-loading*))
     contents
     :multiple-lines t)
    (format t "develenv.cl already correctly loads CG/JS.~%"))
   ((match-re
     (format nil "^#\\+ignore \\(load \"sys:~a"
	     (file-namestring *setup-cgjs-loading*))
     contents
     :multiple-lines t)
    (format t "develenv.cl: CG/JS disabled; will enable.~%")
    (setf (file-contents *develenv.cl*)
      (replace-re  contents
		  (format nil "^#\\+ignore *\\(load \"sys:~a"
			  (file-namestring *setup-cgjs-loading*))
		  (format nil "#||# (load \"sys:~a"
			  (file-namestring *setup-cgjs-loading*))
		  :multiple-lines t
		  :count 1)))
   (t
    (format t "develenv.cl: adding CG/JS initialization.~%")
    (setf (file-contents *develenv.cl*)
      (concatenate 'simple-string
	contents
	(format nil "~

;; To disable the use of CG/JS and go back to the originally released
;; (and possibly patched) version of CG/IDE, change the #||# to
;; #+ignore on the following line and the rebuild images by running
;; update.sh or update.exe, depending on your platform:
#||# (load \"sys:~a\")~%~%"
		(file-namestring *setup-cgjs-loading*)))))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; success!

(let ((msg (format nil "~
CG/JS installation successful!

NOTE: to use CG/JS you must rebuild images by running 'update.~a'
      in the Allegro CL directory (~a)."
		   #+mswindows "exe"
		   #-mswindows "sh"
		   *sys*)))
  #-mswindows (format t "~&~%~a~%" msg)
  #+mswindows (excl::internal-message-box msg "CG/JS Installation" :type :ok))
